library dynamic_middleware;

import 'package:redux/redux.dart';

/// [MiddlewareClass] used to add and remove other [Middleware] dynamically.
/// 
/// Executes other [Middleware<State>] as part of its own middleware [call] method.
class DynamicMiddleware<State> extends MiddlewareClass<State> {
  /// A [Set] of [Middleware<State>] objects with a [call] method.
  final Set<Middleware<State>> middlewares = new Set<Middleware<State>>();

  /// Create a piece of middleware that adds a single piece of [middleware] as dynamic-middleware
  DynamicMiddleware([Middleware<State> middleware]) {
    if(middleware != null) {
      middlewares.add(middleware);
    }
  }

  /// Create a piece of middleware that adds all [middlewareList] as dynamic-middleware
  DynamicMiddleware.fromList(List<Middleware<State>> middlewareList) {
    middlewareList.forEach(middlewares.add);
  }

  @override
  void call(Store<State> store, dynamic action, NextDispatcher next) {
    void middlewareCall(Middleware<State> middleware) => middleware(store, action, next);
    middlewares.forEach(middlewareCall);
    next(action);
  }

  /// Add a single piece of middleware dynamically.
  void add(Middleware<State> middleware) => middlewares.add(middleware);

  /// All a list of middleware dynamically.
  void addAll(List<Middleware<State>> middleware) => middlewares.addAll(middleware);

  /// Remove a piece of middleware dynamically.
  void remove(Middleware<State> middleware) => middlewares.remove(middleware);

  /// Remove all middleware previously added.
  void clear() => middlewares.clear();
}
